/* 
 * File:   Clustering.hpp
 * Author: Thibaut Cuvelier
 * 
 * Implements the K-means clustering algorithm. 
 *
 * Note: This class may perform much better with the use of multithreading
 * (assigning colours to sets may be done concurrently without problem, 
 * as for the computation of the new centroids or the replacement by the
 * optima, as long as concurrent-enabled containers are used, such as those
 * provided by Intel TBB). 
 */

#ifndef CLUSTERING_HPP
#define	CLUSTERING_HPP

#include <vector>
#include "Colour.hpp"
#include "ColourSet.hpp"

using namespace std;

class Clustering {
public:
  Clustering();
  // Clusters values, with a certain number of clusters (level), of which some points can be imposed (startingPoints). The other points will be imposed randomly at first. 
  vector<Colour> clusteredColours(vector<Colour> values, unsigned levels, const vector<Colour> & startingPoints);

private:
  Clustering(const Clustering&);
  void operator=(const Clustering&);

  // Compute an optimum clustering, with the starting point. 
  void computeOptimum(unsigned levels, const vector<Colour> & startingPoints); 
  // Replaces the values by the clustered mapping. 
  void replaceByOptimum(); 

  vector<Colour> values_;
  vector<ColourSet> optimum_; 
};

#endif	/* CLUSTERING_HPP */

