/* 
 * File:   Colour.cpp
 * Author: Thibaut Cuvelier
 */

#include <cstdlib>
#include "Colour.hpp"

using namespace std; 

// Constructors, destructors.
Colour::Colour() : red_(0), green_(0), blue_(0) {}
Colour::Colour(unsigned char red, unsigned char green, unsigned char blue) : red_(red), green_(green), blue_(blue) {}
Colour::~Colour() {}

// Comparators
bool Colour::operator==(const Colour & other) const {
  return red() == other.red() && green() == other.green() && blue() == other.blue(); 
}

bool Colour::operator!=(const Colour & other) const {
  return ! (*this == other);
}

bool Colour::operator<(const Colour & other) const {
  unsigned long hash = 17 + red() + 256 * green() + 256 * 256 * blue(); 
  unsigned long otherHash = 17 + other.red() + 256 * other.green() + 256 * 256 * other.blue(); 
  return hash < otherHash; 
}

// Useful things.
void Colour::fill(istream& in, ostream& out) {
  out << "Red component:" << endl; 
  in >> red_;
  out << "Green component:" << endl;
  in >> green_;
  out << "Blue component:" << endl; 
  in >> blue_;
}

static inline double square(double x) {
  return x * x; 
}

double Colour::squaredEuclideanDistanceTo(const Colour & other) const {
  return square(red() - other.red()) + square(green() - other.green()) + square(blue() - other.blue()); 
}

// Bad quality random numbers (use of modulo may cause the results to lack 
// uniform distribution). Supposes srand() called once. 
static inline unsigned char basicRandom() {
  return rand() % 256; 
}

Colour Colour::randomColour() {
  return Colour(basicRandom(), basicRandom(), basicRandom()); 
}

std::vector<Colour> Colour::fromCharArray(unsigned char* array, unsigned length) { // length not including 3 components (number of pixels)
  std::vector<Colour> v(length);

  for (unsigned i = 0; i < length; ++i) {
    v[i].red(array[3 * i]);
    v[i].green(array[3 * i + 1]);
    v[i].blue(array[3 * i + 2]);
  }

  return v;
}

void Colour::toCharArray(std::vector<Colour> &colours, unsigned char* array) { // array is allocated (3 * length)
  for (long unsigned i = 0; i < colours.size(); ++i) {
    array[3 * i] = colours[i].red();
    array[3 * i + 1] = colours[i].green();
    array[3 * i + 2] = colours[i].blue();
  }
}
 
// Setters
void Colour::red(unsigned char red) { red_ = red; }
void Colour::green(unsigned char green) { green_ = green; }
void Colour::blue(unsigned char blue) { blue_ = blue; }

// Getters
unsigned char Colour::red() const { return red_; }
unsigned char Colour::green() const { return green_; }
unsigned char Colour::blue() const { return blue_; }

ostream & operator<<(ostream & out, const Colour & colour) {
  return out << "(" << colour.red() << ", " << colour.green() << ", " << colour.blue() << ")"; 
}

