/* 
 * File:   Colour.hpp
 * Author: Thibaut Cuvelier
 *
 * A RGB 24-bit-colour. 
 */

#ifndef COLOUR_HPP
#define	COLOUR_HPP

#include <iostream>
#include <vector>

class Colour {
public:
  Colour(); 
  Colour(unsigned char red, unsigned char green, unsigned char blue);
  ~Colour();

  // Operator overloads, required to use colours in STL containers. 
  bool operator==(const Colour & other) const;
  bool operator!=(const Colour & other) const;
  bool operator<(const Colour & other) const; 
  
  // Fills a colour from the provided streams (asks the user for input on ostream and gets the values from istream). 
  void fill(std::istream&, std::ostream&); 
  // Computes the square of the Euclidean distance from this point to the other. 
  double squaredEuclideanDistanceTo(const Colour & other) const; 
  // Outputs a random colour (expects srand(time(NULL)) to be called once). 
  static Colour randomColour(); 
  // Goes from a array like { red1, green1, blue1, red2... } to a vector of colours. 
  static std::vector<Colour> fromCharArray(unsigned char* array, unsigned length);
  // Goes the other way around: from a vector to an array. 
  static void toCharArray(std::vector<Colour> &colours, unsigned char* array);
  
  // Setters.
  void red(unsigned char red);
  void green(unsigned char green);
  void blue(unsigned char blue);
  
  // Getters.
  unsigned char red() const;
  unsigned char green() const;
  unsigned char blue() const;
private:
  unsigned char red_, green_, blue_;
};

// ostream & operator<<(ostream & out, const Colour & colour);

#endif	/* COLOUR_HPP */

