/* 
 * File:   ColourSet.cpp
 * Author: Thibaut Cuvelier
 */

#include <algorithm>
#include "ColourSet.hpp"
#include "Averager.hpp"

using namespace std; 

ColourSet::ColourSet() {}

ColourSet::~ColourSet() {}

bool ColourSet::operator==(const ColourSet& other) {
  return centroid_ == other.centroid_ && equal(colours_.begin(), colours_.end(), other.colours_.begin());
}

bool ColourSet::operator!=(const ColourSet& other) {
  return ! (*this == other);
}

const Colour & ColourSet::centroid() const {
  return centroid_; 
}

void ColourSet::centroid(const Colour & centroid) {
  centroid_ = centroid; 
}

void ColourSet::addColour(const Colour & colour) {
  colours_.push_back(colour); 
}

const std::list<Colour> & ColourSet::colours() const {
  return colours_; 
}

Colour ColourSet::computeNewCentroid() const {
  // If no colour has been added, return current centroid. 
  if(colours_.size() == 0) 
    return centroid(); 
  
  // Otherwise, compute the mean. 
  Averager red, green, blue; 
  for(list<Colour>::const_iterator it = colours_.begin(); it != colours_.end(); ++it) {
    red.add(it->red());
    green.add(it->green());
    blue.add(it->blue()); 
  }
  
  return Colour(red.average(), green.average(), blue.average());
}
