/* 
 * File:   ColourSet.hpp
 * Author: Thibaut Cuvelier
 * 
 * Implements a colour set for the K-means algorithm: a centroid and a list of colours that are close enough to be put in this bucket. 
 */

#ifndef COLOURSET_HPP
#define	COLOURSET_HPP

#include "Colour.hpp"
#include <list>

class ColourSet {
public:
  ColourSet();
  ~ColourSet();

  // Operators for STL containers.
  bool operator==(const ColourSet& other);
  bool operator!=(const ColourSet& other);
  
  // Getter/setter for the centroid of this set. 
  const Colour & centroid() const;
  void centroid(const Colour & centroid);
  // Computes the new centroid, based on the added colours (average of all those colours).
  Colour computeNewCentroid() const;

  // Adds a colour to the list of this set. 
  void addColour(const Colour & colour);
  // Gets the list of colours. 
  const std::list<Colour> & colours() const; 
  
private:
  Colour centroid_; 
  std::list<Colour> colours_; 
};

#endif	/* COLOURSET_HPP */

